/***************************************************************************
 *   Copyright (C) 2004-2011 by Johan Maes -ON4QZ                          *
 *   on4qz@telenet.be   http://users.telenet.be/on4qz                      *
 *   http://users.telenet.be/on4qz                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "location.h"
#include "qgridglobal.h"
#include <QStringList>
#include <QDebug>
/** \file */

double RAD(double angle) {return (angle / 180.0) * M_PI;}
double DEG(double angle) {return (angle / M_PI) * 180.0;}


/*! There are several ways of encoding latitude and longitude:
  - DMS Degrees Minutes Seconds (e.g. 50° 30' 35'')
  - Decimal degrees (e.g. 50.833)
  - ADIF Format: N 050 30.014) i.e. [NSEW] Degrees and decimal minutes.
 Note cty.dat uses the wrong longitude convention, it indicates West as positive instead of negative.
 Internally will use EAST: positive West negative. Conversion will be made in the cty upload module.
*/




/*!
 \param locator QRA locator
 \param latitude  latitude in decimal degrees;
 \param longitude longitude in decimal degrees;
*/
bool locatorToLL(QString locator, double &latitude, double &longitude)
{
	int level;
  level=locator.length();
  locator=locator.toUpper();
    if((level!=4) && (level!=6))return false;

  if (!((locator[0]>='A')&&(locator[0]<='R')&&(locator[1]>='A')&&(locator[1]<='R'))) return false;
  if (!((locator[2]>='0')&&(locator[2]<='9')&&(locator[3]>='0')&&(locator[3]<='9'))) return false;
  if (level==6)
  	{
      if (!((locator[4]>='A')&&(locator[4]<='X')&&(locator[5]>='A')&&(locator[5]<='X'))) return false;
    }
  else
    {
      locator.append("LL");
    }
  longitude = -180.0 + FIELD_WIDTH*(locator[0].toLatin1()-'A') + SQUARE_WIDTH*((int)locator[2].toLatin1()-'0');
  latitude  = -90.0  + FIELD_HEIGHT*(locator[1].toLatin1()-'A') + SQUARE_HEIGHT*(locator[3].toLatin1()-'0');
  longitude+= SUB_WIDTH*(locator[4].toLatin1()-'A')+SUB_WIDTH/2;
  latitude += SUB_HEIGHT*(locator[5].toLatin1()-'A')+SUB_HEIGHT/2;
    return true;
}

/*!
 \param lat1  latitude in decimal degrees;
 \param lon1 longitude in decimal degrees;
 \param locator QRA locator
*/

bool LLToLocator(double lat1, double lon1, QString &locator)
{
	double latitude,longitude;
	int iLatitude, iLongitude;
	latitude=lat1;
  longitude=lon1;
    if (latitude > 90.0 || latitude < -90.0) return false;

    if (longitude > 180.0 || longitude < -180.0) return false;
  locator="";
	// shift range
	latitude  += 90.0;
	longitude += 180.0;

	iLongitude = (int)(longitude / FIELD_WIDTH);
  locator[0]  = iLongitude + 'A';
	longitude -= (double)iLongitude * FIELD_WIDTH;

	iLongitude = (int)(longitude / SQUARE_WIDTH);
  locator[2]  = iLongitude + '0';
	longitude -= (double)iLongitude * SQUARE_WIDTH;

	iLongitude = (int)(longitude / SUB_WIDTH);
  locator[4]  = iLongitude + 'A';

	iLatitude = (int)(latitude / FIELD_HEIGHT);
  locator[1] = iLatitude + 'A';
	latitude -= (double)iLatitude * FIELD_HEIGHT;

	iLatitude = (int)(latitude / SQUARE_HEIGHT);
  locator[3] = iLatitude + '0';
	latitude -= (double)iLatitude * SQUARE_HEIGHT;

	iLatitude = (int)(latitude / SUB_HEIGHT);
  locator[5] = iLatitude + 'A';

    return true;
}

/*!
 \param latitude  latitude in decimal degrees;
 \param longitude longitude in decimal degrees;
 \param strLat latitude in ADIF notation
 \param strLon longitude in ADIF notation
*/


bool LLToADIFString(double latitude, double longitude, QString &strLat,QString &strLon)
{
	QString s;
    if (latitude > 90.0 || latitude < -90.0) return false;
    if (longitude > 180.0 || longitude < -180.0) return false;
	if(latitude<0)
		{
		 strLat="S";
     degreeToADIFString(-latitude,strLat);
		}
	else
		{
			strLat="N";
      degreeToADIFString(latitude,strLat);
		}
	if(longitude>0)
		{
     strLon="E";
      degreeToADIFString(longitude,strLon);
		}
	else
		{
      strLon="W";
      degreeToADIFString(-longitude,strLon);
		}
  return true;
}

/*!
 \param latitude  latitude in decimal degrees;
 \param longitude longitude in decimal degrees;
 \param strLat latitude in ADIF notation
 \param strLon longitude in ADIF notation
*/

bool stringADIFToLL(double &latitude, double &longitude,QString strLat,QString strLon)
{
  QString temp;
  bool negLat=false;
  bool negLon=false;
  strLat=strLat.simplified();
  strLat=strLat.toUpper();
  strLon=strLon.simplified();
  strLon=strLon.toUpper();
  temp=strLat;

  if(strLat[0]=='S')
    {
      negLat=true;
      temp.remove(0,1);
    }
  if(strLat[0]=='N')
    {
      temp.remove(0,1);
    }
  stringADIFToDegree(latitude,temp);
  if (negLat) latitude=-latitude;
  temp=strLon;
  if(strLon[0]=='W')
    {
      negLon=true;
      temp.remove(0,1);
    }
  if(strLon[0]=='E')
    {
      temp.remove(0,1);
    }
  stringADIFToDegree(longitude,temp);
  if (negLon) longitude=-longitude;
  return true;
}

/*!
 \param dg  decimal degrees;
 \param strCoordinate coordinate (lat or lon) in ADIF notation
*/

void degreeToADIFString(double dg,QString &strCoordinate)
{
	double tmp;
  strCoordinate+=QString("%1").arg((int)dg,3,10,QChar('0'));
  strCoordinate+=" ";
	tmp=dg-(int)dg;
	tmp=tmp*60.;
  strCoordinate+=QString("%1").arg(tmp,6,'f',3,QChar('0'));
}

/*!

 \param strCoordinate coordinate (lat or lon) in ADIF notation (e.g. 50 30.300)
 \param dg  decimal degrees;
*/

void stringADIFToDegree(double &dg,QString strCoordinate)
{
  QStringList ll=strCoordinate.split(" ");
  if(ll.count()!=2) return;
	double tmp;
	int d;
  d=ll[0].toInt();
	tmp=ll[1].toDouble();
	tmp=tmp/60+d;
	dg=tmp;
}

void stringToDM_Degree(QString &result,QString s)
{
  double tmp,tmp2;
  tmp=s.toDouble();
  result=QString::number((int)tmp,10);
  result+=" ";
  tmp2=tmp-(int)tmp;
  result+=QString::number(tmp2*60);
}



/*!
  Given the latitude and longitude of two locations returns the distance and the bearing.

 \param lat1 latitude in decimal degrees;
 \param lon1 longitude in decimal degrees;
 \param lat2 latitude in decimal degrees;
 \param lon2 longitude in decimal degrees;
 \param bearing direction in decimal degrees
 \param distance  distance in km between the two points.
*/

void calcDistanceBearing(double lat1, double lon1, double lat2, double lon2, double &bearing, double &distance)
{
	double si, co, ca;	/* Locals */
	lat1 = RAD(lat1);
	lat2 = RAD(lat2);
  lon1 = RAD(lon1);
  lon2 = RAD(lon2);

	distance=dist(lat1,lon1,lat2,lon2);

	co = cos(lon2-lon1)*cos(lat2)*cos(lat1) + sin(lat2)*sin(lat1);
	ca = acos(co);

  if(useMiles)
      {
				distance=distance/1.609344;
      }

    // If the distance is reasonable, calculate the
    // bearing. Otherwise return it as zero.
    if (distance > 1e-1)
      {
	    	si = sin(lon1-lon2)*cos(lat1)*cos(lat2);
	 			co = sin(lat1) - sin(lat2)*cos(ca);

				bearing = atan2((si > 0 ? si : -si), (co > 0 ? co : -co));
				if (co < 0) bearing = M_PI - bearing;
				if (si < 0)	bearing = -bearing;
				if (bearing < 0) bearing = bearing + 2*M_PI;
				bearing *= 360/(2*M_PI);
      }
    else
      {
				bearing = 0;
      } 
} 


/*!
  Given the latitude and longitude of two locations returns the distance

 \param lat1 latitude in decimal degrees;
 \param lon1 longitude in decimal degrees;
 \param lat2 latitude in decimal degrees;
 \param lon2 longitude in decimal degrees;
 \return distance in km between the two points.
*/


double dist(double lat1, double lon1, double lat2,double lon2)
{

  double F = (lat1 + lat2) / 2.0;
  double G = (lat1 - lat2) / 2.0;
  double L = (lon1 - lon2) / 2.0;

  double sing = sin(G);
  double cosl = cos(L);
  double cosf = cos(F);
  double sinl = sin(L);
  double sinf = sin(F);
  double cosg = cos(G);

  double S = sing*sing*cosl*cosl + cosf*cosf*sinl*sinl;
  double C = cosg*cosg*cosl*cosl + sinf*sinf*sinl*sinl;
  double W = atan2(sqrt(S),sqrt(C));
	if(W==0) return 0;
  double R = sqrt((S*C))/W;
  double H1 = (3 * R - 1.0) / (2.0 * C);
  double H2 = (3 * R + 1.0) / (2.0 * S);
  double D = 2 * W * ERAD;
  return (D * (1 + FLATTENING * H1 * sinf*sinf*cosg*cosg - FLATTENING*H2*cosf*cosf*sing*sing));
}


double convertFromDMS(int d,int m, int s)
{
  return((double)d+(double)m/60.+(double)s/3600.);
}

void convertToDMS(double x, int &d,int &m, int &s)
{
  double st;
  st=(x>0 ? x+1./7200. : x -1./7200.); // round 1/2 of 1/3600
  d=(int)st;
  st=60*(st-(double)d);
  m=(int)st;
  st=60*(st-(double)m);
  s=(int)st;
}



