/***************************************************************************
 *   Copyright (C) 2004-2011 by Johan Maes -ON4QZ                          *
 *   on4qz@telenet.be   http://users.telenet.be/on4qz                      *
 *   http://users.telenet.be/on4qz                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "qgridglobal.h"
#include <QPainter>
#include <QSettings>
#include <QCloseEvent>
#include <QMessageBox>
#include "location.h"
#include <QDebug>
#include "ui_configdialog.h"

static const QPoint needle[4] =
{
  QPoint(0,100),
  QPoint(-10,0),
  QPoint(0, -100),
  QPoint(10,0)
};

static const QPoint needleTop[3] =
{
  QPoint(0,-100),
  QPoint(-3,-70),
  QPoint(3, -70)
};


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
   ui->setupUi(this);
   compassPixmap.load(":/icons/compass.xpm");
   side=compassPixmap.width();
   needleColor=Qt::yellow;
   needleTopColor=Qt::red;
//   connect(ui->milesCheckBox,SIGNAL(toggled(bool)),SLOT(slotMilesChanged(bool)));
   connect(ui->locHome,SIGNAL(returnPressed()),SLOT(slotLocHomeChanged()));
   connect(ui->actionConfigure,SIGNAL(triggered()),SLOT(slotConfigure()));
   connect(ui->actionExit,SIGNAL(triggered()),SLOT(slotExit()));
   connect(ui->clearPushButton,SIGNAL(clicked()),SLOT(slotClear()));
   connect(ui->LLToLocatorPushButton,SIGNAL(clicked()),SLOT(slotLLToLocator()));
   connect(ui->LocatorToLLPushButton,SIGNAL(clicked()),SLOT(slotLocatorToLL()));
   connect(ui->actionAbout_QGrid,SIGNAL(triggered()),SLOT(slotAboutQGrid()));
   connect(ui->actionAbout_Qt,SIGNAL(triggered()),SLOT(slotAboutQt()));
   connect(ui->locRemote,SIGNAL(returnPressed()),SLOT(slotLocRemoteChanged()));
   readSettings();
   drawCompass(0);
   setParams();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::readSettings()
{
  QSettings qSettings(ORGANIZATION,APPLICATION);
  qSettings.beginGroup(APP_KEY);
  int windowWidth = qSettings.value("windowWidth", 700 ).toInt();
  int windowHeight = qSettings.value("windowHeight", 480 ).toInt();
  int windowX = qSettings.value( "windowX", -1 ).toInt();
  int windowY = qSettings.value( "windowY", -1 ).toInt();
  useMiles= qSettings.value( "useMiles", 0 ).toBool();
  useDMS= qSettings.value( "useDMS", 1 ).toBool();
  homeLocator=qSettings.value( "homeLocator", "").toString();
  resize( windowWidth, windowHeight );
  if ( windowX != -1 || windowY != -1 ) 	move( windowX, windowY );
  qSettings.endGroup();
}

void MainWindow::writeSettings()
{
  QSettings qSettings(ORGANIZATION,APPLICATION);
  qSettings.beginGroup(APP_KEY);
  qSettings.setValue( "windowWidth", width() );
  qSettings.setValue( "windowHeight", height() );
  qSettings.setValue( "windowX", x() );
  qSettings.setValue( "windowY", y() );
  qSettings.setValue( "useMiles", useMiles );
  qSettings.setValue( "useDMS", useDMS );
  qSettings.setValue( "homeLocator",homeLocator);
  qSettings.endGroup();
}

void MainWindow::slotExit()
{
  writeSettings();
  qApp->quit();
}


void  MainWindow::closeEvent(QCloseEvent *event)
{
  writeSettings();
  event->accept();
}

void MainWindow::setParams()
{
  ui->locHome->setText(homeLocator);
}

void MainWindow::slotClear()
{
  ui->locHome->setText(homeLocator);
  ui->locRemote->setText("");
  ui->latHomeDegrees->setText("");
  ui->latHomeMinutes->setText("");
  ui->latHomeSeconds->setText("");
  ui->latRemoteDegrees->setText("");
  ui->latRemoteMinutes->setText("");
  ui->latRemoteSeconds->setText("");

  ui->longHomeDegrees->setText("");
  ui->longHomeMinutes->setText("");
  ui->longHomeSeconds->setText("");
  ui->longRemoteDegrees->setText("");
  ui->longRemoteMinutes->setText("");
  ui->longRemoteSeconds->setText("");
  ui->distance->setText("");
  ui->bearing->setText("");
}

void MainWindow::slotLLToLocator()
{
  double latRemote,latHome,longHome,longRemote;
  int dla,mla,sla,dlb,mlb,slb,dea,mea,sea,deb,meb,seb;
  QString temp;
  if(useDMS)
    {
      dla=ui->latHomeDegrees->text().toInt();
      mla=ui->latHomeMinutes->text().toInt();
      sla=ui->latHomeSeconds->text().toInt();
      if(dla<0)
        {
          mla=-mla; sla=-sla;
        }
      dlb=ui->latRemoteDegrees->text().toInt();
      mlb=ui->latRemoteMinutes->text().toInt();
      slb=ui->latRemoteSeconds->text().toInt();
      if(dlb<0)
        {
          mlb=-mlb; slb=-slb;
        }
      dea=ui->longHomeDegrees->text().toInt();
      mea=ui->longHomeMinutes->text().toInt();
      sea=ui->longHomeSeconds->text().toInt();
      if(dea<0)
        {
          mea=-mea; sea=-sea;
        }
      deb=ui->longRemoteDegrees->text().toInt();
      meb=ui->longRemoteMinutes->text().toInt();
      seb=ui->longRemoteSeconds->text().toInt();
      if(deb<0)
        {
          meb=-meb; seb=-seb;
        }
      latHome=convertFromDMS(dla,mla,sla);
      latRemote=convertFromDMS(dlb,mlb,slb);
      longHome=convertFromDMS(dea,mea,sea);
      longRemote=convertFromDMS(deb,meb,seb);
    }
  else
    {
      latHome=ui->latHomeDegrees->text().toDouble();
      longHome=ui->longHomeDegrees->text().toDouble();
      latRemote=ui->latRemoteDegrees->text().toDouble();
      longRemote=ui->longRemoteDegrees->text().toDouble();
    }

  LLToLocator(latHome,longHome,temp);
  ui->locHome->setText(temp);
  LLToLocator(latRemote,longRemote,temp);
  ui->locRemote->setText(temp);

  double be,dist;
  calcDistanceBearing(latRemote,longRemote,latHome,longHome,be,dist);
  setInt(ui->distance,(int)(dist+0.5));
  setInt(ui->bearing,(int)(be+0.5));
  drawCompass((int)(be+0.5));
}

void MainWindow::slotLocatorToLL()
{

  int  d,m,s;
  double latHome,longHome;
  double latRemote,longRemote;
  double dist,be;
  bool homeOK=false,remoteOK=false;
  if( validateLocator(ui->locHome))
   {
      homeOK=true;
      homeLocator=ui->locHome->text();;
      locatorToLL(homeLocator,latHome, longHome);
      convertToDMS(latHome,d,m,s);
      if(useDMS)
        {
          setInt(ui->latHomeDegrees,d);
          setInt(ui->latHomeMinutes,(m>0?m:-m));
          setInt(ui->latHomeSeconds,(s>0?s:-s));
        }
      else
        {
         setDouble(ui->latHomeDegrees,latHome);
          ui->latHomeMinutes->clear();
          ui->latHomeSeconds->clear();
        }
      if(useDMS)
        {
          convertToDMS(longHome,d,m,s);
          setInt(ui->longHomeDegrees,d);
          setInt(ui->longHomeMinutes,(m>0?m:-m));
          setInt(ui->longHomeSeconds,(s>0?s:-s));
        }
      else
        {
         setDouble(ui->longHomeDegrees,longHome);
          ui->longHomeMinutes->clear();
          ui->longHomeSeconds->clear();
        }
   }
  if( validateLocator(ui->locRemote))
   {
      remoteOK=true;
      remoteLocator=ui->locRemote->text();;
      locatorToLL(remoteLocator,latRemote, longRemote);
      if(useDMS)
        {
          convertToDMS(latRemote,d,m,s);
          setInt(ui->latRemoteDegrees,d);
          setInt(ui->latRemoteMinutes,(m>0?m:-m));
          setInt(ui->latRemoteSeconds,(s>0?s:-s));
        }
      else
        {
         setDouble(ui->latRemoteDegrees,latRemote);
          ui->latRemoteMinutes->clear();
          ui->latRemoteSeconds->clear();
        }
      if(useDMS)
        {
          convertToDMS(longRemote,d,m,s);
          setInt(ui->longRemoteDegrees,d);
          setInt(ui->longRemoteMinutes,(m>0?m:-m));
          setInt(ui->longRemoteSeconds,(s>0?s:-s));
        }
      else
        {
         setDouble(ui->longRemoteDegrees,longRemote);
          ui->longRemoteMinutes->clear();
          ui->longRemoteSeconds->clear();
        }
   }
  if (homeOK && remoteOK)
    {
      calcDistanceBearing(latRemote,longRemote,latHome,longHome,be,dist);
      setInt(ui->distance,(int)(dist+0.5));
      setInt(ui->bearing,(int)(be+0.5));
      drawCompass((int)(be+0.5));
    }

}

void MainWindow::drawCompass(double degrees)
{
    QPixmap comboPix=compassPixmap;
    QPainter painter(&comboPix);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.translate(side / 2, side / 2);
    painter.scale(side /200.0, side / 200.0);
    painter.setBrush(needleColor);
    painter.rotate(degrees);
    painter.drawConvexPolygon(needle, 4);
    painter.setBrush(needleTopColor);
    painter.drawConvexPolygon(needleTop, 3);
    ui->compassLabel->setPixmap(comboPix);
}

void  MainWindow::slotMilesChanged(bool inMiles)
{
   useMiles=inMiles;
}

void MainWindow::slotLocHomeChanged()
{
  if(validateLocator(ui->locHome))
  {
    homeLocator=ui->locHome->text();
   slotLocatorToLL();
  }
}

void MainWindow::slotLocRemoteChanged()
{
 if(validateLocator(ui->locRemote))
  {
   remoteLocator=ui->locRemote->text();
   slotLocatorToLL();
  }
}

bool  MainWindow::validateLocator(QLineEdit *le)
{
  QString tmp;
  tmp=le->text().toUpper();
  if( validateLocator(tmp))
    {
      le->setText(tmp);

      le->setStyleSheet("background-color: white;");
      return true;
    }
  else le->setStyleSheet("background-color: red;");
  return false;
}

bool  MainWindow::validateLocator(const QString loc)
{
  if (loc.length()<4)  return false;
  if ((loc[0]>='A')&&(loc[0]<='R')&&(loc[1]>='A')&&(loc[1]<='R'))
    {
      if(loc[2].isDigit()&&loc[3].isDigit())
        {
          if (loc.length()==4) return true;
          else if ((loc[4]>='A')&&(loc[4]<='X')&&(loc[5]>='A')&&(loc[5]<='X')) return true;
        }
    }
  return false;
}

void MainWindow::slotConfigure()
{
  QDialog dg;
  Ui::configDialog cd;
  cd.setupUi(&dg);
  cd.useMilesCheckBox->setChecked(useMiles);
  cd.useDMSCheckBox->setChecked(useDMS);
  dg.exec();
  useMiles=cd.useMilesCheckBox->isChecked();
  useDMS=cd.useDMSCheckBox->isChecked();
}

void MainWindow::slotAboutQGrid()
{
  QString t;
  t=qgrid_version+"\n\n";
  t+="QGrid allows you to input either the QRA locator or Latitude/Longitude to compute the bearing and the distance between your "
    "home location and a remote station.\n\n"
    "You can enter the QRA locator for your current position and use the \"To Lat/long\" button to calculate the Latitude and Longitude.\n"
    "\nIf you enter Lat/Long information in the correct format (see below) then you can use the \"To Locator\" button.\n"
    "The home locator will be saved in between sessions.\n"
    "\nThere are two option under Files->Configure:\n"
    "Miles:      activate this to have the display in miles\n"
    "DMS Lat/Long Format:   If activated, the display of Latitude and Longitude will be in Degrees, Minutes, Seconds. Otherwise it will be in decimal degrees.\n"
    "You have to use the selected format when entering data in the Lat/long fields\n"
    "\nThe backgroundcolour of the QRA locator will change to red when an invalid (or blank) locator is entered.";

  QMessageBox::information(this,"QGrid Help",t);
}
void MainWindow::slotAboutQt()
{
  QMessageBox::aboutQt(this);
}

void MainWindow::setInt(QLineEdit *ql,int x)
{
  ql->setText(QString::number(x));
}

void MainWindow::setDouble(QLineEdit *ql,double x)
{
  ql->setText(QString::number(x));
}
